@@ -3,8 +3,6 @@ try:
 except NameError:
     basestring = (str, bytes)
 
-from .patch import Version as _Version
-
 from .structures import (
     PyteomicsError, Charge, ChargeList,
     _parse_charge, _parse_ion, BasicComposition,
@@ -33,5 +31,5 @@ from .target_decoy import (
     _log_pi, _make_fdr, fdr, sigma_T, sigma_fdr)
 
 from .utils import (
-    print_tree, memoize, BinaryDataArrayTransformer,
-    _decode_base64_data_array)
+    print_tree, memoize, BinaryDataArrayTransformer, ArrayConversionMixin, BinaryArrayConversionMixin,
+    MaskedArrayConversionMixin, _decode_base64_data_array)
