@@ -393,14 +393,18 @@ class IndexedTextReader(IndexedReaderMixin, FileReader):
     label = None
     block_size = 1000000
     label_group = 1
+    _kw_keys = ['delimiter', 'label', 'block_size', 'label_group']
+    _warn_if_empty = True
 
     def __init__(self, source, **kwargs):
         # the underlying _file_obj gets None as encoding
         # to avoid transparent decoding of StreamReader on read() calls
         encoding = kwargs.pop('encoding', 'utf-8')
+        if 'warn_if_empty' in kwargs:
+            self._warn_if_empty = kwargs.pop('warn_if_empty')
         super(IndexedTextReader, self).__init__(source, mode='rb', encoding=None, **kwargs)
         self.encoding = encoding
-        for attr in ['delimiter', 'label', 'block_size', 'label_group']:
+        for attr in self._kw_keys:
             if attr in kwargs:
                 setattr(self, attr, kwargs.pop(attr))
         self._offset_index = None
@@ -410,11 +414,16 @@ class IndexedTextReader(IndexedReaderMixin, FileReader):
     def __getstate__(self):
         state = super(IndexedTextReader, self).__getstate__()
         state['offset_index'] = self._offset_index
+        for key in self._kw_keys:
+            state[key] = getattr(self, key)
         return state
 
     def __setstate__(self, state):
         super(IndexedTextReader, self).__setstate__(state)
         self._offset_index = state['offset_index']
+        for key in self._kw_keys:
+            if key in state:
+                setattr(self, key, state[key])
 
     def _chunk_iterator(self):
         fh = self._source.file
@@ -473,6 +482,8 @@ class IndexedTextReader(IndexedReaderMixin, FileReader):
             last_label = label
             last_offset = offset
         assert last_label is None
+        if self._warn_if_empty and not index:
+            self._warn_empty()
         return index
 
     def _read_lines_from_offsets(self, start, end):
@@ -480,6 +491,10 @@ class IndexedTextReader(IndexedReaderMixin, FileReader):
         lines = self._source.read(end - start).decode(self.encoding).split('\n')
         return lines
 
+    def _warn_empty(self):
+        warnings.warn("{} object has an empty index for file {}. If this is unexpected, consider adjusting `label`.".format(
+            self.__class__.__name__, getattr(self._source, 'name', self._source_init)))
+
 
 class IndexSavingMixin(NoOpBaseReader):
     """Common interface for :py:class:`IndexSavingXML` and :py:class:`IndexSavingTextReader`."""
@@ -529,25 +544,24 @@ class IndexSavingMixin(NoOpBaseReader):
             self._offset_index.save(f)
 
     @_keepstate_method
-    def _build_index(self):
+    def build_byte_index(self):
         """Build the byte offset index by either reading these offsets
         from the file at :attr:`_byte_offset_filename`, or falling back
-        to the method used by :class:`IndexedXML` if this operation fails
+        to the method used by :class:`IndexedXML` or :class:`IndexedTextReader` if this operation fails
         due to an IOError
         """
-        if not self._use_index: return
+        if not getattr(self, '_use_index', True): return  # indexed text readers do not have `_use_index`
         try:
-            self._read_byte_offsets()
+            return self._read_byte_offsets()
         except (IOError, AttributeError, TypeError):
-            super(IndexSavingMixin, self)._build_index()
+            return super(IndexSavingMixin, self).build_byte_index()
 
     def _read_byte_offsets(self):
         """Read the byte offset index JSON file at :attr:`_byte_offset_filename`
         and populate :attr:`_offset_index`
         """
         with open(self._byte_offset_filename, 'r') as f:
-            index = self._index_class.load(f)
-            self._offset_index = index
+            return self._index_class.load(f)
 
 
 def _file_reader(_mode='r'):
@@ -570,30 +584,19 @@ def _file_reader(_mode='r'):
     return decorator
 
 
-def _file_writer(_mode='a'):
+def _file_writer(_mode='w'):
     def decorator(_func):
         """A decorator that opens output files for writer functions.
         """
         @wraps(_func)
         def helper(*args, **kwargs):
-            if 'file_mode' in kwargs:
-                m = kwargs.pop('file_mode')
-                warn = False
-            else:
-                m = _mode
-                warn = True
+            m = kwargs.pop('file_mode', _mode)
             enc = kwargs.pop('encoding', None)
             if len(args) > 1:
                 out_arg = args[1]
             else:
                 out_arg = kwargs.pop('output', None)
 
-            # warn about the change in default mode if an existing file name is given
-            if isinstance(out_arg, basestring) and warn and os.path.exists(out_arg):
-                warnings.warn("Opening an existing file in append mode. "
-                    "The default mode will change from 'a' to 'w' in a future version. "
-                    "Pass `file_mode='a'` to keep old behavior and suppress this warning.", FutureWarning)
-
             with _file_obj(out_arg, m, encoding=enc) as out:
                 if len(args) > 1:
                     call_args = (args[0], out) + args[2:]
@@ -1144,7 +1147,7 @@ class ChainBase(object):
     file names or file objects. Keyword arguments are passed to
     the :meth:`sequence_maker` function.
 
-    Attributes
+    Parameters
     ----------
     sources : :class:`Iterable`
         Sources for creating new sequences from, such as paths or
@@ -1166,11 +1169,13 @@ class ChainBase(object):
     def _make_chain(cls, sequence_maker):
         if isinstance(sequence_maker, type):
             tp = type('%sChain' % sequence_maker.__class__.__name__, (cls,), {
-                'sequence_maker': sequence_maker
+                'sequence_maker': sequence_maker,
+                '__doc__': cls.__doc__.replace(':meth:`sequence_maker`', ':class:`{}`'.format(sequence_maker.__name__))
             })
         else:
             tp = type('FunctionChain', (cls,), {
-                'sequence_maker': staticmethod(sequence_maker)
+                'sequence_maker': staticmethod(sequence_maker),
+                '__doc__': cls.__doc__.replace(':meth:`sequence_maker`', ':func:`{}`'.format(sequence_maker.__name__))
             })
         return tp
 
