@@ -14,11 +14,13 @@ except ImportError:
     from collections import Container, Sized
 from bisect import bisect_right
 from contextlib import contextmanager
-
+try:
+    import pandas as pd
+except ImportError:
+    pd = None
 
 from .structures import PyteomicsError
 from .file_helpers import _keepstate, IteratorContextManager, _make_chain, ChainBase, TableJoiner
-from .patch import pd
 
 
 def _fix_docstring(f, **defaults):
