@@ -112,11 +112,40 @@ import itertools
 import random
 from collections import namedtuple
 import re
+import abc
 from . import auxiliary as aux
+from .auxiliary.utils import add_metaclass
 
 
 Protein = namedtuple('Protein', ('description', 'sequence'))
 DECOY_PREFIX = 'DECOY_'
+RAW_HEADER_KEY = '__raw__'
+
+
+def _add_raw_field(parser):
+    """
+    Add :py:const:`RAW_HEADER_KEY` field to the parsed dictinary.
+
+    Parameters
+    ----------
+    parser : func
+        parser function.
+
+    Returns
+    -------
+    None.
+
+    """
+    def _new_parser(instance, descr):
+        parsed = parser(instance, descr)
+        if RAW_HEADER_KEY not in parsed:
+            parsed[RAW_HEADER_KEY] = descr
+        elif parsed[RAW_HEADER_KEY] != descr:
+            raise aux.PyteomicsError('Cannot save raw protein header, since the corresponsing'
+                                    'key ({}) already exists.'.format(RAW_HEADER_KEY))
+        return parsed
+
+    return _new_parser
 
 
 class FASTABase(object):
@@ -386,10 +415,61 @@ class TwoLayerIndexedFASTA(IndexedFASTA):
                     return super(TwoLayerIndexedFASTA, self).get_entry(header)
         raise KeyError(key)
 
+    def get_header(self, key):
+        if key in self._id2header:
+            return self._id2header[key]
+        raise KeyError(key)
+
     def __contains__(self, key):
         return super(TwoLayerIndexedFASTA, self).__contains__(key) or key in self._id2header
 
 
+class _FastaParserFlavorMeta(abc.ABCMeta):
+    def __new__(mcs, name, bases, namespace):
+        if "parser" in namespace:
+            namespace["parser"] = _add_raw_field(namespace["parser"])
+        if name != 'FlavoredMixin':
+            reader_type = None
+            for t in (FASTA, IndexedFASTA, TwoLayerIndexedFASTA):
+                if t in bases:
+                    reader_type = t
+
+            if reader_type is not None:
+                # this is a "concrete" reader class
+                # add a unified __init__ method for it
+                for c in bases:
+                    if issubclass(c, FlavoredMixin):
+                        flavor = c
+                        break
+                else:
+                    raise aux.PyteomicsError('Could not detect flavor of {}, not a subclass of `FlavoredMixin`.')
+
+                def __init__(self, source, parse=True, **kwargs):
+                    reader_type.__init__(self, source, **kwargs)
+                    flavor.__init__(self, parse)
+                    self._init_args = (source, parse)
+                    self._init_kwargs = kwargs
+
+                flavor_name = name[:-5]
+                type_name = "Text-mode" if reader_type is FASTA else "Indexed"
+                __init__.__doc__ = """Creates a :py:class:`{}` object.
+
+                Parameters
+                ----------
+                source : str or file
+                    The file to read. If a file object, it needs to be in *{}* mode.
+                parse : bool, optional
+                    Defines whether the descriptions should be parsed in the produced tuples.
+                    Default is :py:const:`True`.
+                kwargs : passed to the :py:class:`{}` constructor.
+                """.format(name, 'text' if reader_type is FASTA else 'binary', reader_type.__name__)
+                namespace['__init__'] = __init__
+                namespace['__doc__'] = """{} parser for {} FASTA files.""".format(type_name, flavor_name)
+
+        return super(_FastaParserFlavorMeta, mcs).__new__(mcs, name, bases, namespace)
+
+
+@add_metaclass(_FastaParserFlavorMeta)
 class FlavoredMixin():
     """Parser aimed at a specific FASTA flavor.
     Subclasses should define `parser` and `header_pattern`.
@@ -402,80 +482,45 @@ class FlavoredMixin():
 
 
 class UniProtMixin(FlavoredMixin):
-    header_pattern = r'^(\w+)\|([-\w]+)\|(\w+)\s+([^=]*\S)((\s+\w+=[^=]+(?!\w*=))+)\s*$'
-    header_group = 2
+    header_pattern = r'^(?P<db>\w+)\|(?P<id>[-\w]+)\|(?P<entry>\w+)\s+(?P<name>.*?)(?:(\s+OS=(?P<OS>[^=]+))|(\s+OX=(?P<OX>\d+))|(\s+GN=(?P<GN>[^=]+))|(\s+PE=(?P<PE>\d))|(\s+SV=(?P<SV>\d+)))*\s*$'
+    header_group = 'id'
 
     def parser(self, header):
-        db, ID, entry, name, pairs, _ = re.match(self.header_pattern, header).groups()
-        gid, taxon = entry.split('_')
-        info = {'db': db, 'id': ID, 'entry': entry,
-                'name': name, 'gene_id': gid, 'taxon': taxon}
-        info.update(_split_pairs(pairs))
-        _intify(info, ('PE', 'SV'))
+        info = re.match(self.header_pattern, header).groupdict()
+        for key in ['OS', 'OX', 'GN', 'PE', 'SV']:
+            if info[key] is None:
+                del info[key]
+        info['gene_id'], info['taxon'] = info['entry'].split('_')
+        _intify(info, ('PE', 'SV', 'OX'))
         return info
 
 
-def _add_init(cls):
-    """Add an __init__ method to a flavored parser class,
-    which simply calls __init__ of its two bases."""
-    flavor, typ = cls.__bases__
-    newdict = cls.__dict__.copy()
-    def __init__(self, source, parse=True, **kwargs):
-        typ.__init__(self, source, **kwargs)
-        flavor.__init__(self, parse)
-        self._init_args = (source, parse)
-        self._init_kwargs = kwargs
-
-    flavor_name = flavor.__name__[:-5]
-    type_name = "Text-mode" if typ is FASTA else "Indexed"
-    __init__.__doc__ = """Creates a :py:class:`{}` object.
-
-    Parameters
-    ----------
-    source : str or file
-        The file to read. If a file object, it needs to be in *{}* mode.
-    parse : bool, optional
-        Defines whether the descriptions should be parsed in the produced tuples.
-        Default is :py:const:`True`.
-    kwargs : passed to the :py:class:`{}` constructor.
-    """.format(cls.__name__, 'text' if typ is FASTA else 'binary', typ.__name__)
-    newdict['__init__'] = __init__
-    newdict['__doc__'] = """{} parser for {} FASTA files.""".format(type_name, flavor_name)
-
-    return type(cls.__name__, (flavor, typ), newdict)
-
-
-@_add_init
 class UniProt(UniProtMixin, FASTA):
     pass
 
 
-@_add_init
 class IndexedUniProt(UniProtMixin, TwoLayerIndexedFASTA):
     pass
 
 
 class UniRefMixin(FlavoredMixin):
-    header_pattern = r'^(\S+)\s+([^=]*\S)((\s+\w+=[^=]+(?!\w*=))+)\s*$'
+    header_pattern = r'^(?P<id>\S+)\s+(?P<cluster>.*?)(?:(\s+n=(?P<n>\d+))|(\s+Tax=(?P<Tax>.+?))|(\s+TaxID=(?P<TaxID>\S+))|(\s+RepID=(?P<RepID>\S+)))*\s*$'
+    header_group = 'id'
 
     def parser(self, header):
         assert 'Tax' in header
-        ID, cluster, pairs, _ = re.match(self.header_pattern, header).groups()
-        info = {'id': ID, 'cluster': cluster}
-        info.update(_split_pairs(pairs))
-        gid, taxon = info['RepID'].split('_')
-        type_, acc = ID.split('_')
-        info.update({'taxon': taxon, 'gene_id': gid, 'type': type_, 'accession': acc})
+        info = re.match(self.header_pattern, header).groupdict()
+        for key in ['TaxID', 'Tax', 'RepID', 'n']:
+            if info[key] is None:
+                del info[key]
         _intify(info, ('n',))
         return info
 
 
-@_add_init
 class UniRef(UniRefMixin, FASTA):
     pass
 
 
-@_add_init
 class IndexedUniRef(UniRefMixin, TwoLayerIndexedFASTA):
     pass
 
@@ -488,12 +533,10 @@ class UniParcMixin(FlavoredMixin):
         return {'id': ID, 'status': status}
 
 
-@_add_init
 class UniParc(UniParcMixin, FASTA):
     pass
 
 
-@_add_init
 class IndexedUniParc(UniParcMixin, TwoLayerIndexedFASTA):
     pass
 
@@ -510,12 +553,10 @@ class UniMesMixin(FlavoredMixin):
         return info
 
 
-@_add_init
 class UniMes(UniMesMixin, FASTA):
     pass
 
 
-@_add_init
 class IndexedUniMes(UniMesMixin, TwoLayerIndexedFASTA):
     pass
 
@@ -530,12 +571,10 @@ class SPDMixin(FlavoredMixin):
                 'taxon': taxon, 'gene_id': gid}
 
 
-@_add_init
 class SPD(SPDMixin, FASTA):
     pass
 
 
-@_add_init
 class IndexedSPD(SPDMixin, TwoLayerIndexedFASTA):
     pass
 
@@ -548,12 +587,10 @@ class NCBIMixin(FlavoredMixin):
         return {'id': ID, 'description': description, 'taxon': organism}
 
 
-@_add_init
 class NCBI(NCBIMixin, FASTA):
     pass
 
 
-@_add_init
 class IndexedNCBI(NCBIMixin, TwoLayerIndexedFASTA):
     pass
 
@@ -566,12 +603,10 @@ class RefSeqMixin(FlavoredMixin):
         return {'id': ID, 'description': description, 'taxon': organism}
 
 
-@_add_init
 class RefSeq(RefSeqMixin, FASTA):
     pass
 
 
-@_add_init
 class IndexedRefSeq(RefSeqMixin, TwoLayerIndexedFASTA):
     pass
 
@@ -617,15 +652,26 @@ def write(entries, output=None):
 
     Parameters
     ----------
-    entries : iterable of (str, str) tuples
+    entries : iterable of (str/dict, str) tuples
         An iterable of 2-tuples in the form (description, sequence).
+        If description is a dictionary, it must have a special key, whose value
+        will be written as protein description. The special key is defined by the variable
+        :py:const:`RAW_HEADER_KEY`.
     output : file-like or str, optional
         A file open for writing or a path to write to. If the file exists,
-        it will be opened for appending. Default is :py:const:`None`, which
+        it will be opened for writing. Default is :py:const:`None`, which
         means write to standard output.
+
+        .. note::
+            The default mode for output files specified by name has been changed
+            from `a` to `w` in *pyteomics 4.6*. See `file_mode` to override the mode.
+
     file_mode : str, keyword only, optional
         If `output` is a file name, defines the mode the file will be opened in.
-        Otherwise will be ignored. Default is 'a'.
+        Otherwise will be ignored. Default is `'w'`.
+
+        .. note ::
+            The default changed from `'a'` in *pyteomics 4.6*.
 
     Returns
     -------
@@ -633,7 +679,12 @@ def write(entries, output=None):
         The file where the FASTA is written.
     """
     for descr, seq in entries:
-        output.write('>' + descr.replace('\n', '\n;') + '\n')
+        if isinstance(descr, str):
+            output.write('>' + descr.replace('\n', '\n;') + '\n')
+        elif isinstance(descr, dict) and RAW_HEADER_KEY in descr:
+            output.write('>' + descr[RAW_HEADER_KEY].replace('\n', '\n;') + '\n')
+        else:
+             raise aux.PyteomicsError('Cannot use provided description: ' + repr(descr))
         output.write(''.join([('%s\n' % seq[i:i+70])
             for i in range(0, len(seq), 70)]) + '\n')
 
@@ -698,20 +749,20 @@ def shuffle(sequence, keep_nterm=False, keep_cterm=False, keep_nterm_M=False, fi
     # empty sequence
     if len(sequence) == 0:
         return ''
-    
+
     # presereve the first position
     if (keep_nterm_M and sequence[0] == 'M') or keep_nterm:
-        return sequence[0] + shuffle(sequence[1:], keep_cterm=keep_cterm, 
+        return sequence[0] + shuffle(sequence[1:], keep_cterm=keep_cterm,
                        fix_aa=fix_aa)
-    
+
     # presereve the last position
     if keep_cterm:
         return shuffle(sequence[:-1], fix_aa=fix_aa) + sequence[-1]
-    
-    
+
+
     if not isinstance(fix_aa, str):
         fix_aa = ''.join(fix_aa)
-    
+
     fixed = []
     position = 0
     if len(fix_aa) > 0:  # non-empty fixed list
@@ -721,15 +772,15 @@ def shuffle(sequence, keep_nterm=False, keep_cterm=False, keep_nterm_M=False, fi
             shuffled.extend(sequence[position:match.start()])
             position = match.end()
         shuffled.extend(sequence[position:])
-        
+
     else:  # shuffle everything
         shuffled = list(sequence)
-    
+
     random.shuffle(shuffled)
-    
+
     for fix in fixed:
         shuffled.insert(fix[0], fix[1])
-    
+
     return ''.join(shuffled)
 
 
