@@ -148,6 +148,11 @@ _atom = r'([A-Z][a-z+]*)(?:\[(\d+)\])?([+-]?\d+)?'
 _formula = r'^({})*$'.format(_atom)
 
 
+def _raise_term_label_exception(what='comp'):
+    raise PyteomicsError("Cannot use a mod label as a terminal group. Provide correct group {0}"
+                         " in `aa_{0}`.".format(what))
+
+
 class Composition(BasicComposition):
     """
     A Composition object stores a chemical composition of a
@@ -164,10 +169,20 @@ class Composition(BasicComposition):
     def _from_parsed_sequence(self, parsed_sequence, aa_comp):
         self.clear()
         comp = defaultdict(int)
+        failflag = False
         for label in parsed_sequence:
             if label in aa_comp:
                 for elem, cnt in aa_comp[label].items():
                     comp[elem] += cnt
+            elif parser.is_term_group(label):
+                slabel = label.strip('-')
+                if slabel in aa_comp:
+                    # a modification label used as terminal group. This is prone to errors and not allowed
+                    _raise_term_label_exception()
+                elif re.match(_formula, slabel):
+                    comp += Composition(formula=slabel)
+                else:
+                    failflag = True
             else:
                 try:
                     mod, aa = parser._split_label(label)
@@ -176,7 +191,9 @@ class Composition(BasicComposition):
                         comp[elem] += cnt
 
                 except (PyteomicsError, KeyError):
-                    raise PyteomicsError('No information for %s in `aa_comp`' % label)
+                    failflag = True
+            if failflag:
+                raise PyteomicsError('No information for %s in `aa_comp`' % label)
         self._from_composition(comp)
 
     def _from_split_sequence(self, split_sequence, aa_comp):
@@ -186,15 +203,16 @@ class Composition(BasicComposition):
             i = 0
             while i < len(group):
                 for j in range(len(group) + 1, -1, -1):
-                    try:
-                        label = ''.join(group[i:j])
+                    label = ''.join(group[i:j])
+                    if label in aa_comp:
                         for elem, cnt in aa_comp[label].items():
                             comp[elem] += cnt
-                    except KeyError:
-                        continue
+                    elif parser.is_term_group(label) and label.strip('-') in aa_comp:
+                        _raise_term_label_exception()
                     else:
-                        i = j
-                        break
+                        continue
+                    i = j
+                    break
                 if j == 0:
                     raise PyteomicsError("Invalid group starting from position %d: %s" % (i + 1, group))
         self._from_composition(comp)
@@ -202,7 +220,6 @@ class Composition(BasicComposition):
     def _from_sequence(self, sequence, aa_comp):
         parsed_sequence = parser.parse(
             sequence,
-            labels=aa_comp,
             show_unmodified_termini=True)
         self._from_parsed_sequence(parsed_sequence, aa_comp)
 
@@ -255,8 +272,7 @@ class Composition(BasicComposition):
         Parameters
         ----------
         formula : str, optional
-            A string with a chemical formula. All elements must be present in
-            `mass_data`.
+            A string with a chemical formula.
         sequence : str, optional
             A polypeptide sequence string in modX notation.
         parsed_sequence : list of str, optional
@@ -367,6 +383,51 @@ class Composition(BasicComposition):
             charge = int(charge) * (-1, 1)[sign == '+']
         return comp, charge
 
+    @staticmethod
+    def _mass_to_mz(mass, composition=None, **kwargs):
+        mass_data = kwargs.get('mass_data', nist_mass)
+        absolute = kwargs.get('absolute', True)
+        average = kwargs.get('average', False)
+
+        # Calculate m/z if required
+        charge = kwargs.get('charge')
+        if charge:
+            # get charge carrier mass and charge
+            charge_carrier = kwargs.get('charge_carrier')
+            ccharge = kwargs.get('carrier_charge')
+            if isinstance(charge_carrier, dict):
+                carrier_comp = Composition(charge_carrier)
+                if ccharge and PROTON in carrier_comp:
+                    raise PyteomicsError('`carrier_charge` specified but the charge carrier contains protons.')
+                carrier_charge = ccharge or carrier_comp[PROTON]
+                if not carrier_charge:
+                    raise PyteomicsError('Charge carrier charge not specified.')
+            else:
+                carrier_comp, carrier_charge = (composition or Composition)._parse_carrier(charge_carrier)
+                if carrier_charge and ccharge:
+                    raise PyteomicsError('Both `carrier_charge` and charge in carrier spec are given.')
+                carrier_charge = ccharge or carrier_charge
+                if not carrier_charge:
+                    raise PyteomicsError('Charge of the charge carrier group not specified.')
+            if charge % carrier_charge:
+                raise PyteomicsError('The `charge` must be a multiple of the carrier charge. Given: {} and {}'.format(
+                    charge, carrier_charge))
+            num = charge // carrier_charge
+            carrier_mass = carrier_comp.mass(mass_data=mass_data, average=average, charge=0)
+
+        if charge and (composition is None or not composition['H+']):
+            mass += carrier_mass * num
+        if charge and composition and composition['H+']:
+            raise PyteomicsError('Composition contains protons and charge is explicitly specified.')
+        if charge is None and composition and composition['H+']:
+            warnings.warn('Charge is not specified, but the Composition contains protons. Assuming m/z calculation.')
+            charge = composition['H+']
+        if charge:
+            mass /= charge
+        if charge and charge < 0 and absolute:
+            mass = abs(mass)
+        return mass
+
     def mass(self, **kwargs):
         """Calculate the mass or *m/z* of a :py:class:`Composition`.
 
@@ -408,25 +469,24 @@ class Composition(BasicComposition):
             If specified, then the polypeptide is considered to be in the form
             of the corresponding ion. Do not forget to specify the charge state!
         absolute : bool, optional
-            If :py:const:`True`, the m/z value returned will always be positive,
+            If :py:const:`True` (default), the m/z value returned will always be positive,
             even for negatively charged ions.
 
-            .. warning::
-                Default is :py:const:`False` now, but will be changed in a future version.
+            .. note ::
+                `absolute` only applies when `charge` is negative.
+                The mass can still be negative for negative compositions.
 
         Returns
         -------
         mass : float
         """
-        composition = self
         mass_data = kwargs.get('mass_data', nist_mass)
 
         # Calculate mass
         mass = 0.0
         average = kwargs.get('average', False)
-        absolute = kwargs.get('absolute')
 
-        for isotope_string, amount in composition.items():
+        for isotope_string, amount in self.items():
             element_name, isotope_num = _parse_isotope_string(isotope_string)
             # Calculate average mass if required and the isotope number is
             # not specified.
@@ -437,46 +497,7 @@ class Composition(BasicComposition):
             else:
                 mass += (amount * mass_data[element_name][isotope_num][0])
 
-        # Calculate m/z if required
-        charge = kwargs.get('charge')
-        if charge:
-            # get charge carrier mass and charge
-            charge_carrier = kwargs.get('charge_carrier')
-            ccharge = kwargs.get('carrier_charge')
-            if isinstance(charge_carrier, dict):
-                carrier_comp = Composition(charge_carrier)
-                if ccharge and PROTON in carrier_comp:
-                    raise PyteomicsError('`carrier_charge` specified but the charge carrier contains protons.')
-                carrier_charge = ccharge or carrier_comp[PROTON]
-                if not carrier_charge:
-                    raise PyteomicsError('Charge carrier charge not specified.')
-            else:
-                carrier_comp, carrier_charge = self._parse_carrier(charge_carrier)
-                if carrier_charge and ccharge:
-                    raise PyteomicsError('Both `carrier_charge` and charge in carrier spec are given.')
-                carrier_charge = ccharge or carrier_charge
-                if not carrier_charge:
-                    raise PyteomicsError('Charge of the charge carrier group not specified.')
-            if charge % carrier_charge:
-                raise PyteomicsError('The `charge` must be a multiple of the carrier charge. Given: {} and {}'.format(
-                    charge, carrier_charge))
-            num = charge // carrier_charge
-            carrier_mass = carrier_comp.mass(mass_data=mass_data, average=average, charge=0)
-        if charge and not composition['H+']:
-            mass += carrier_mass * num
-        if charge and composition['H+']:
-            raise PyteomicsError('Composition contains protons and charge is explicitly specified.')
-        if charge is None and composition['H+']:
-            warnings.warn('Charge is not specified, but the Composition contains protons. Assuming m/z calculation.')
-            charge = composition['H+']
-        if charge:
-            mass /= charge
-        if mass < 0 and absolute is None:
-            warnings.warn('Returning a signed value. The default will change in the future.'
-                ' Specify `absolute` kwarg to suppress this warning', FutureWarning)
-        if absolute:
-            mass = abs(mass)
-        return mass
+        return self._mass_to_mz(mass, self, **kwargs)
 
 
 std_aa_comp.update({
@@ -567,6 +588,9 @@ def calculate_mass(*args, **kwargs):
         A string with a chemical formula.
     sequence : str, optional
         A polypeptide sequence string in modX notation.
+    proforma : str, optional
+        A polypeptide sequeence string in `ProForma notation <https://www.psidev.info/proforma>`_,
+        or a :py:class:`pyteomics.proforma.ProForma` object.
     parsed_sequence : list of str, optional
         A polypeptide sequence parsed into a list of amino acids.
     composition : Composition, optional
@@ -615,26 +639,37 @@ def calculate_mass(*args, **kwargs):
         If specified, then the polypeptide is considered to be in the form
         of the corresponding ion. Do not forget to specify the charge state!
     absolute : bool, optional
-            If :py:const:`True`, the m/z value returned will always be positive,
-            even for negatively charged ions.
+        If :py:const:`True` (default), the m/z value returned will always be positive,
+        even for negatively charged ions.
 
-            .. warning::
-                Default is :py:const:`False` now, but will be changed in a future version.
+        .. note ::
+            `absolute` only applies when `charge` is negative.
+            The mass can still be negative for negative compositions.
 
     Returns
     -------
     mass : float
     """
-    # These parameters must not be passed to mass(), not __init__
-    charge = kwargs.pop('charge', None)
-    charge_carrier = kwargs.pop('charge_carrier', None)
-    carrier_charge = kwargs.pop('carrier_charge', None)
-    absolute = kwargs.pop('absolute', None)
+    if 'proforma' in kwargs:
+        # do not try to create a composition
+        from .. import proforma
+        proteoform = kwargs.pop('proforma')
+        if isinstance(proteoform, str):
+            proteoform = proforma.ProForma.parse(proteoform)
+        return Composition._mass_to_mz(proteoform.mass, **kwargs)
+
+    # These parameters must be passed to mass(), not __init__
+    mass_kw = {}
+    for k in ['charge', 'charge_carrier', 'carrier_charge', 'absolute']:
+        if k in kwargs:
+            mass_kw[k] = kwargs.pop(k)
     # Make a copy of `composition` keyword argument.
-    composition = (Composition(kwargs['composition']) if 'composition' in kwargs else Composition(*args, **kwargs))
-    return composition.mass(
-        charge=charge, charge_carrier=charge_carrier, carrier_charge=carrier_charge,
-        absolute=absolute, **kwargs)
+    if 'composition' in kwargs:
+        composition = Composition(kwargs.pop('composition'))
+    else:
+        composition = Composition(*args, **kwargs)
+    kwargs.update(mass_kw)
+    return composition.mass(**kwargs)
 
 
 def most_probable_isotopic_composition(*args, **kwargs):
@@ -952,7 +987,7 @@ def fast_mass2(sequence, ion_type=None, charge=None, **kwargs):
         value is :py:data:`nist_mass`).
     aa_mass : dict, optional
         A dict with the monoisotopic mass of amino acid residues
-        (default is std_aa_mass);
+        (default is std_aa_mass).
     ion_comp : dict, optional
         A dict with the relative elemental compositions of peptide ion
         fragments (default is :py:data:`std_ion_comp`).
@@ -980,7 +1015,11 @@ def fast_mass2(sequence, ion_type=None, charge=None, **kwargs):
                 mass += aa_mass[aa] * num
             elif parser.is_term_mod(aa):
                 assert num == 1
-                mass += calculate_mass(formula=aa.strip('-'), mass_data=mass_data)
+                group = aa.strip('-')
+                if group in aa_mass:
+                    _raise_term_label_exception('mass')
+                else:
+                    mass += calculate_mass(formula=group, mass_data=mass_data)
             else:
                 mod, X = parser._split_label(aa)
                 mass += (aa_mass[mod] + aa_mass[X]) * num
