@@ -35,8 +35,7 @@ Classes
 Functions
 ---------
 
-  :py:func:`read` - iterate through spectra in MGF file. Data from a
-  single spectrum are converted to a human-readable dict.
+  :py:func:`read` - an alias for :py:class:`MGF` or :py:class:`IndexedMGF`.
 
   :py:func:`get_spectrum` - read a single spectrum with given title from a file.
 
@@ -73,21 +72,13 @@ except ImportError:
     np = None
 import itertools as it
 import sys
+import warnings
 from . import auxiliary as aux
 
 
-class MGFBase(object):
+class MGFBase(aux.MaskedArrayConversionMixin):
     """Abstract mixin class representing an MGF file. Subclasses implement different approaches to parsing."""
     _comments = set('#;!/')
-    _array = (lambda x, dtype: np.array(x, dtype=dtype)) if np is not None else None
-    _ma = (lambda x, dtype: np.ma.masked_equal(np.array(x, dtype=dtype), 0)) if np is not None else None
-    _identity = lambda x, **kw: x
-    _array_converters = {
-        'm/z array': [_identity, _array, _array],
-        'intensity array': [_identity, _array, _array],
-        'charge array': [_identity, _array, _ma],
-        'ion array': [_identity, _array,  _array]
-    }
     _array_keys = ['m/z array', 'intensity array', 'charge array', 'ion array']
     _array_keys_unicode = [u'm/z array', u'intensity array', u'charge array', u'ion array']
     encoding = None
@@ -108,7 +99,7 @@ class MGFBase(object):
             Default is :py:const:`True`.
 
         convert_arrays : one of {0, 1, 2}, optional, keyword only
-            If `0`, m/z, intensities and (possibly) charges or (possibly) ions will be returned as regular lists
+            If `0`, m/z, intensities and (possibly) charges or (possibly) ions will be returned as regular lists.
             If `1`, they will be converted to regular :py:class:`numpy.ndarray`'s.
             If `2`, charges will be reported as a masked array (default).
             The default option is the slowest. `1` and `2` require :py:mod:`numpy`.
@@ -130,25 +121,53 @@ class MGFBase(object):
 
         super(MGFBase, self).__init__(source, **kwargs)
         self._use_header = kwargs.pop('use_header', True)
-        self._convert_arrays = kwargs.pop('convert_arrays', 2)
-        if self._convert_arrays and np is None:
-            raise aux.PyteomicsError('numpy is required for array conversion')
         self._read_charges = kwargs.pop('read_charges', True)
         self._read_ions = kwargs.pop('read_ions', False)
         # Make sure no charges are read if ions are read
         if self._read_ions:
             self._read_charges = False
-        dtype = kwargs.pop('dtype', None)
-        self._dtype_dict = dtype if isinstance(dtype, dict) else {k: dtype for k in self._array_keys}
         if self._use_header:
             self._read_header()
         else:
             self._header = None
 
+    def __reduce_ex__(self, protocol):
+        return (self.__class__, (self._source_init,), self.__getstate__())
+
+    def __getstate__(self):
+        state = super(MGFBase, self).__getstate__()
+        state['use_header'] = self._use_header
+        state['header'] = self._header
+        return state
+
+    def __setstate__(self, state):
+        super(MGFBase, self).__setstate__(state)
+        self._header = state['header']
+        self._use_header = state['use_header']
+
     @staticmethod
     def parse_precursor_charge(charge_text, list_only=False):
         return aux._parse_charge(charge_text, list_only=list_only)
 
+    @staticmethod
+    def parse_pepmass_charge(pepmass_str):
+        split = pepmass_str.split()
+        if len(split) > 3:
+            raise aux.PyteomicsError('MGF format error: cannot parse '
+                    'PEPMASS = {}'.format(pepmass_str))
+        elif len(split) == 3:
+            charge = split[2]
+            try:
+                pepmass = tuple(map(float, split[:2]))
+            except ValueError:
+                raise aux.PyteomicsError('MGF format error: cannot parse '
+                    'PEPMASS = {}'.format(pepmass_str))
+        else:
+            pepmass = tuple(map(float, split[:2]))
+            pepmass = pepmass + (None,) * (2-len(pepmass))
+            charge = None
+        return pepmass, charge
+
     @staticmethod
     def parse_peak_charge(charge_text, list_only=False):
         return aux._parse_charge(charge_text, list_only=False)
@@ -203,25 +222,19 @@ class MGFBase(object):
                 pass
             elif sline == 'END IONS':
                 if 'pepmass' in params:
-                    try:
-                        pepmass = tuple(map(float, params['pepmass'].split()))
-                    except ValueError:
-                        raise aux.PyteomicsError('MGF format error: cannot parse '
-                                'PEPMASS = {}'.format(params['pepmass']))
-                    else:
-                        params['pepmass'] = pepmass + (None,) * (2-len(pepmass))
+                    params['pepmass'], charge = self.parse_pepmass_charge(params['pepmass'])
+                    if charge is not None:
+                        params['charge'] = charge
                 if isinstance(params.get('charge'), aux.basestring):
                     params['charge'] = self.parse_precursor_charge(params['charge'], True)
                 if 'rtinseconds' in params:
                     params['rtinseconds'] = aux.unitfloat(params['rtinseconds'], 'second')
-                out = {'params': params}
-                data = {'m/z array': masses, 'intensity array': intensities}
+                out = {'params': params, 'm/z array': masses, 'intensity array': intensities}
                 if self._read_charges:
-                    data['charge array'] = charges
+                    out['charge array'] = charges
                 if self._read_ions:
-                    data['ion array'] = ions
-                for key, values in data.items():
-                    out[key] = self._array_converters[key][self._convert_arrays](values, dtype=self._dtype_dict.get(key))
+                    out['ion array'] = ions
+                self._build_all_arrays(out)
                 if self.encoding and sys.version_info.major == 2:
                     for key, ukey in zip(self._array_keys + ['params'], self._array_keys_unicode + [u'params']):
                         if key in out:
@@ -273,7 +286,6 @@ class IndexedMGF(MGFBase, aux.TaskMappingMixin, aux.TimeOrderedIndexedReaderMixi
     and 'params' stores a :py:class:`dict` of parameters (keys and values are
     :py:class:`str`, keys corresponding to MGF, lowercased).
 
-
     Attributes
     ----------
 
@@ -286,6 +298,54 @@ class IndexedMGF(MGFBase, aux.TaskMappingMixin, aux.TimeOrderedIndexedReaderMixi
 
     def __init__(self, source=None, use_header=True, convert_arrays=2, read_charges=True,
                  dtype=None, encoding='utf-8', index_by_scans=False, read_ions=False, _skip_index=False, **kwargs):
+        """
+        Create an :py:class:`IndexedMGF` (binary-mode) reader for a given MGF file.
+
+        Parameters
+        ----------
+
+        source : str or file or None, optional
+            A file object (or file name) with data in MGF format. Default is
+            :py:const:`None`, which means read standard input.
+
+            .. note :: If a file object is given, it must be opened in binary mode.
+
+        use_header : bool, optional
+            Add the info from file header to each dict. Spectrum-specific parameters
+            override those from the header in case of conflict.
+            Default is :py:const:`True`.
+
+        convert_arrays : one of {0, 1, 2}, optional
+            If `0`, m/z, intensities and (possibly) charges will be returned as regular lists.
+            If `1`, they will be converted to regular :py:class:`numpy.ndarray`'s.
+            If `2`, charges will be reported as a masked array (default).
+            The default option is the slowest. `1` and `2` require :py:mod:`numpy`.
+
+        read_charges : bool, optional
+            If `True` (default), fragment charges are reported. Disabling it improves performance.
+
+        read_ions : bool, optional
+            If `True` (default: False), fragment ion types are reported. Disabling it improves performance.
+            Note that right now, only one of (read_charges, read_ions) may be True.
+
+        dtype : type or str or dict, optional
+            dtype argument to :py:mod:`numpy` array constructor, one for all arrays or one for each key.
+            Keys should be 'm/z array', 'intensity array', 'charge array' and/or 'ion array'.
+
+        encoding : str, optional
+            File encoding.
+
+        block_size : int, optinal
+            Size of the chunk (in bytes) used to parse the file when creating the byte offset index.
+
+        Returns
+        -------
+
+        out : IndexedMGF
+            The reader object.
+        """
+        self._index_by_scans = index_by_scans
+        self._read_ions = read_ions
         self.label = r'SCANS=(\d+)\s*' if index_by_scans else r'TITLE=([^\n]*\S)\s*'
         super(IndexedMGF, self).__init__(source, parser_func=self._read, pass_file=False, args=(), kwargs={},
                                          use_header=use_header, convert_arrays=convert_arrays,
@@ -296,22 +356,9 @@ class IndexedMGF(MGFBase, aux.TaskMappingMixin, aux.TimeOrderedIndexedReaderMixi
     def __reduce_ex__(self, protocol):
         return (self.__class__,
                 (self._source_init, False, self._convert_arrays, self._read_charges,
-                 self._dtype_dict, self.encoding, True),
+                 None, self.encoding, self._index_by_scans, self._read_ions, True),
                 self.__getstate__())
 
-    def __getstate__(self):
-        state = super(IndexedMGF, self).__getstate__()
-        state['use_header'] = self._use_header
-        state['header'] = self._header
-        state['read_ions'] = self._read_ions
-        return state
-
-    def __setstate__(self, state):
-        super(IndexedMGF, self).__setstate__(state)
-        self._header = state['header']
-        self._use_header = state['use_header']
-        self._read_ions = state['read_ions']
-
     @aux._keepstate_method
     def _read_header(self):
         try:
@@ -334,6 +381,11 @@ class IndexedMGF(MGFBase, aux.TaskMappingMixin, aux.TimeOrderedIndexedReaderMixi
     def get_spectrum(self, key):
         return self.get_by_id(key)
 
+    def _warn_empty(self):
+        text = ("{} object has an empty index for file {}. If this is unexpected, consider adjusting `label` or "
+            "setting `index_by_scans={}`.".format(
+            self.__class__.__name__, getattr(self._source, 'name', self._source_init), not self._index_by_scans))
+        warnings.warn(text)
 
 class MGF(MGFBase, aux.FileReader):
     """
@@ -361,10 +413,52 @@ class MGF(MGFBase, aux.FileReader):
 
     def __init__(self, source=None, use_header=True, convert_arrays=2, read_charges=True,
             read_ions=False, dtype=None, encoding=None):
+        """
+        Create an :py:class:`MGF` (text-mode) reader for a given MGF file.
+
+        Parameters
+        ----------
+
+        source : str or file or None, optional
+            A file object (or file name) with data in MGF format. Default is
+            :py:const:`None`, which means read standard input.
+
+            ..note :: If a file object is given, it must be opened in text mode.
+
+        use_header : bool, optional
+            Add the info from file header to each dict. Spectrum-specific parameters
+            override those from the header in case of conflict.
+            Default is :py:const:`True`.
+
+        convert_arrays : one of {0, 1, 2}, optional
+            If `0`, m/z, intensities and (possibly) charges will be returned as regular lists.
+            If `1`, they will be converted to regular :py:class:`numpy.ndarray`'s.
+            If `2`, charges will be reported as a masked array (default).
+            The default option is the slowest. `1` and `2` require :py:mod:`numpy`.
+
+        read_charges : bool, optional
+            If `True` (default), fragment charges are reported. Disabling it improves performance.
+
+        read_ions : bool, optional
+            If `True` (default: False), fragment ion types are reported. Disabling it improves performance.
+            Note that right now, only one of (read_charges, read_ions) may be True.
+
+        dtype : type or str or dict, optional
+            dtype argument to :py:mod:`numpy` array constructor, one for all arrays or one for each key.
+            Keys should be 'm/z array', 'intensity array', 'charge array' and/or 'ion array'.
+
+        encoding : str, optional
+            File encoding.
+
+        Returns
+        -------
+
+        out : MGF
+            The reader object.
+        """
         super(MGF, self).__init__(source, mode='r', parser_func=self._read, pass_file=False, args=(), kwargs={},
             encoding=encoding, use_header=use_header, convert_arrays=convert_arrays, read_charges=read_charges,
             read_ions=read_ions, dtype=dtype)
-        # self.encoding = encoding
 
     @aux._keepstate_method
     def _read_header(self):
@@ -419,7 +513,7 @@ def read(*args, **kwargs):
         If `True` (default), fragment charges are reported. Disabling it improves performance.
 
     read_ions : bool, optional
-        If `True` (default: False), fragment charges are reported. Disabling it improves performance.
+        If `True` (default: False), fragment ion types are reported. Disabling it improves performance.
         Note that right now, only one of (read_charges, read_ions) may be True.
 
     dtype : type or str or dict, optional
@@ -535,7 +629,11 @@ def _pepmass_repr(k, pepmass):
 
 
 def _charge_repr(k, charge):
-    return '{}={}'.format(k.upper(), aux._parse_charge(str(charge)))
+    try:
+        val = aux.Charge(charge)
+    except (TypeError, aux.PyteomicsError):
+        val = aux.ChargeList(charge)
+    return '{}={}'.format(k.upper(), val)
 
 
 def _default_repr(key, val):
@@ -555,7 +653,7 @@ def write(spectra, output=None, header='', key_order=_default_key_order, fragmen
     ----------
 
     spectra : iterable
-        A sequence of dictionaries with keys 'm/z array', 'intensity array',
+        A **sequence** of dictionaries with keys 'm/z array', 'intensity array',
         and 'params'. 'm/z array' and 'intensity array' should be sequences of
         :py:class:`int`, :py:class:`float`, or :py:class:`str`. Strings will
         be written 'as is'. The sequences should be of equal length, otherwise
@@ -566,14 +664,25 @@ def write(spectra, output=None, header='', key_order=_default_key_order, fragmen
         without any format consistency tests. Values can be of any type allowing
         string representation.
 
-        'charge array' can also be specified.
+        'charge array' or 'ion array' can also be specified.
+
+        .. note ::
+            Passing a single spectrum will work, but will trigger a warning. This usage pattern is discouraged.
+            To ensure correct output when writing multiple spectra,
+            it is recommended to construct a sequence of spectra first and then call :py:func:`write` once.
+
+        .. seealso ::
+            This discussion of usage patterns of :py:func:`write`: https://github.com/levitsky/pyteomics/discussions/109
 
     output : str or file or None, optional
         Path or a file-like object open for writing. If an existing file is
-        specified by file name, it will be opened for appending. In this case
-        writing with a header can result in violation of format conventions.
+        specified by file name, it will be opened for writing.
         Default value is :py:const:`None`, which means using standard output.
 
+        .. note::
+            The default mode for output files specified by name has been changed
+            from `a` to `w` in *pyteomics 4.6*. See `file_mode` to override the mode.
+
     header : dict or (multiline) str or list of str, optional
         In case of a single string or a list of strings, the header will be
         written 'as is'. In case of dict, the keys (must be strings) will be
@@ -629,7 +738,10 @@ def write(spectra, output=None, header='', key_order=_default_key_order, fragmen
 
     file_mode : str, keyword only, optional
         If `output` is a file name, defines the mode the file will be opened in.
-        Otherwise will be ignored. Default is 'a'.
+        Otherwise will be ignored. Default is `'w'`.
+
+        .. note ::
+            The default changed from `'a'` in *pyteomics 4.6*.
 
     encoding : str, keyword only, optional
         Output file encoding (if `output` is specified by name).
@@ -679,6 +791,12 @@ def write(spectra, output=None, header='', key_order=_default_key_order, fragmen
     if head_str:
         output.write(head_str + '\n\n')
 
+    if isinstance(spectra, dict) and 'm/z array' in spectra:
+        spectra = (spectra, )
+        warnings.warn("Passing a single spectrum to `write()` is discouraged. "
+            "To write a set of spectra, pass them to `write()` all at once. "
+            "For more info, see: https://github.com/levitsky/pyteomics/discussions/109.")
+
     for spectrum in spectra:
         output.write('BEGIN IONS\n')
         found = set()
