@@ -6,23 +6,32 @@ Summary
 -------
 
 `MS1 <http://dx.doi.org/10.1002/rcm.1603>`_ is a simple
-human-readable format for MS1 data. It allows storing MS1 peak lists and
-exprimental parameters.
+human-readable format for MS1 data. It allows storing MS1 peak lists and exprimental parameters.
 
-This module provides minimalistic infrastructure for access to data stored in
-MS1 files.
+This module provides minimalistic infrastructure for access to data stored in MS1 files.
 Two main classes are :py:class:`MS1`, which provides an iterative, text-mode parser,
 and :py:class:`IndexedMS1`, which is a binary-mode parser that supports random access using scan IDs
 and retention times.
 The function :py:func:`read` helps dispatch between the two classes.
-Also, common parameters can be read from MS1 file header with
-:py:func:`read_header` function.
+Also, common parameters can be read from MS1 file header with :py:func:`read_header` function.
+
+Classes
+-------
+
+  :py:class:`MS1` - a text-mode MS1 parser. Suitable to read spectra from a file consecutively.
+  Needs a file opened in text mode (or will open it if given a file name).
+
+  :py:class:`IndexedMS1` - a binary-mode MS1 parser. When created, builds a byte offset index
+  for fast random access by spectrum ID. Sequential iteration is also supported.
+  Needs a seekable file opened in binary mode (if created from existing file object).
+
+  :py:class:`MS1Base` - abstract class, the common ancestor of the two classes above.
+  Can be used for type checking.
 
 Functions
 ---------
 
-  :py:func:`read` - iterate through spectra in MS1 file. Data from a
-  single spectrum are converted to a human-readable dict.
+  :py:func:`read` - an alias for :py:class:`MS1` or :py:class:`IndexedMS1`.
 
   :py:func:`chain` - read multiple files at once.
 
@@ -56,16 +65,43 @@ except ImportError:
     np = None
 
 
-class MS1Base(object):
+class MS1Base(aux.ArrayConversionMixin):
     """Abstract class representing an MS1 file. Subclasses implement different approaches to parsing."""
     _array_keys = ['m/z array', 'intensity array']
+    _float_keys = ['RTime', 'RetTime']
 
-    def __init__(self, source=None, use_header=False, convert_arrays=True, dtype=None, **kwargs):
-        super(MS1Base, self).__init__(source, **kwargs)
+    def __init__(self, source=None, use_header=False, convert_arrays=True, dtype=None, encoding=None, **kwargs):
+        """
+        Create an instance of a :py:class:`MS1Base` parser.
+
+        Parameters
+        ----------
+
+        source : str or file or None, optional
+            A file object (or file name) with data in MS1 format. Default is
+            :py:const:`None`, which means read standard input.
+
+        use_header : bool, optional
+            Add the info from file header to each dict. Spectrum-specific parameters
+            override those from the header in case of conflict.
+            Default is :py:const:`False`.
+
+        convert_arrays : one of {0, 1, 2}, optional
+            If `0`, m/z, intensities and (possibly) charges will be returned as regular lists.
+            If `1`, they will be converted to regular :py:class:`numpy.ndarray`'s.
+            If `2`, charges will be reported as a masked array (default).
+            The default option is the slowest. `1` and `2` require :py:mod:`numpy`.
+
+        dtype : type or str or dict, optional
+            dtype argument to :py:mod:`numpy` array constructor, one for all arrays or one for each key.
+            Keys should be 'm/z array', 'intensity array', 'charge array'.
+
+        encoding : str, optional
+            File encoding.
+        """
+        super(MS1Base, self).__init__(source, use_header=use_header, convert_arrays=convert_arrays, dtype=dtype, encoding=encoding, **kwargs)
         if convert_arrays and np is None:
             raise aux.PyteomicsError('numpy is required for array conversion')
-        self._convert_arrays = convert_arrays
-        self._dtype_dict = dtype if isinstance(dtype, dict) else {k: dtype for k in self._array_keys}
         self._use_header = use_header
         if use_header:
             self._header = self._read_header()
@@ -73,6 +109,9 @@ class MS1Base(object):
             self._header = None
         self._source_name = getattr(source, 'name', str(source))
 
+    def reset(self):
+        super(MS1Base, self).reset()
+        self._pending_line = None
 
     @property
     def header(self):
@@ -91,18 +130,12 @@ class MS1Base(object):
             header[key] = val
         return header
 
-    def _make_scan(self, params, masses, intensities):
-        if 'RTime' in params:
-            params['RTime'] = float(params['RTime'])
-        out = {'params': params}
-        if self._convert_arrays:
-            data = {'m/z array': masses, 'intensity array': intensities}
-            for key, values in data.items():
-                out[key] = np.array(values, dtype=self._dtype_dict.get(key))
-        else:
-            out['m/z array'] = masses
-            out['intensity array'] = intensities
-        return out
+    def _make_scan(self, info):
+        for key in self._float_keys:
+            if key in info['params']:
+                info['params'][key] = float(info['params'][key])
+        self._build_all_arrays(info)
+        return info
 
     def _handle_S(self, line, sline, params):
         sline = line.strip().split(None, 3)
@@ -111,7 +144,7 @@ class MS1Base(object):
             params['precursor m/z'] = float(sline[3])
 
     def _handle_I(self, line, sline, params):
-        params[sline[1]] = sline[2]
+        params[sline[1]] = sline[2] if len(sline) > 2 else ''
 
     def _handle_Z(self, line, sline, params):
         params.setdefault('charge', []).append(float(sline[1]))
@@ -120,15 +153,31 @@ class MS1Base(object):
     def _handle_D(self, line, sline, params):
         params.setdefault('analyzer', []).append(sline[1:])
 
+    def _handle_peak(self, line, sline, info):
+        try:
+            info['m/z array'].append(float(sline[0]))            # this may cause
+            info['intensity array'].append(float(sline[1]))      # exceptions...
+        except ValueError:
+            raise aux.PyteomicsError(
+                'Error when parsing %s. Line: %s' % (self._source_name, line))
+        except IndexError:
+            pass
+
     def _read_spectrum_lines(self, lines):
-        reading_spectrum = False
         params = {}
-        masses = []
-        intensities = []
+        info = {'params': params}
+        for k in self._array_keys:
+            info[k] = []
         if self._use_header:
             params.update(self.header)
-
-        for line in lines:
+        if self._pending_line:
+            reading_spectrum = True
+            self._handle_S(self._pending_line, None, params)
+        else:
+            reading_spectrum = False
+        line_count = 0
+        for i, line in enumerate(lines):
+            line_count = i
             sline = line.strip().split(None, 2)
             if not sline:
                 continue
@@ -141,7 +190,8 @@ class MS1Base(object):
                 if not sline:
                     pass
                 elif sline[0] == 'S':
-                    return self._make_scan(params, masses, intensities)
+                    self._pending_line = line
+                    return self._make_scan(info)
 
                 else:
                     if sline[0] == 'I':  # spectrum-specific parameters!
@@ -151,16 +201,27 @@ class MS1Base(object):
                     elif sline[0] == 'D':  # MS2-specific analyzer annotation
                         self._handle_D(line, sline, params)
                     else:  # this must be a peak list
-                        try:
-                            masses.append(float(sline[0]))            # this may cause
-                            intensities.append(float(sline[1]))       # exceptions...\
-                        except ValueError:
-                            raise aux.PyteomicsError(
-                                'Error when parsing %s. Line: %s' % (
-                                    self._source_name, line))
-                        except IndexError:
-                            pass
-        return self._make_scan(params, masses, intensities)
+                        self._handle_peak(line, sline, info)
+        self._pending_line = None
+        if line_count == 0:
+            return
+        return self._make_scan(info)
+
+    def __getstate__(self):
+        state = super(MS1Base, self).__getstate__()
+        state['use_header'] = self._use_header
+        state['header'] = self._header
+        return state
+
+    def __setstate__(self, state):
+        super(MS1Base, self).__setstate__(state)
+        self._use_header = state['use_header']
+        self._header = state['header']
+
+    def __reduce_ex__(self, protocol):
+        return (self.__class__,
+            (self._source_init, False, self._convert_arrays, None, self.encoding),
+            self.__getstate__())
 
 
 class MS1(MS1Base, aux.FileReader):
@@ -182,63 +243,55 @@ class MS1(MS1Base, aux.FileReader):
 
     """
     def __init__(self, source=None, use_header=False, convert_arrays=True, dtype=None, encoding=None, **kwargs):
+        """
+        Create an :py:class:`MS1` (text-mode) reader for a given MS1 file.
+
+        Parameters
+        ----------
+
+        source : str or file or None, optional
+            A file object (or file name) with data in MS1 format. Default is
+            :py:const:`None`, which means read standard input.
+
+            .. note :: If a file object is given, it must be opened in text mode.
+
+        use_header : bool, optional
+            Add the info from file header to each dict. Spectrum-specific parameters
+            override those from the header in case of conflict.
+            Default is :py:const:`False`.
+
+        convert_arrays : one of {0, 1, 2}, optional
+            If `0`, m/z, intensities and (possibly) charges will be returned as regular lists.
+            If `1`, they will be converted to regular :py:class:`numpy.ndarray`'s.
+            If `2`, charges will be reported as a masked array (default).
+            The default option is the slowest. `1` and `2` require :py:mod:`numpy`.
+
+        dtype : type or str or dict, optional
+            dtype argument to :py:mod:`numpy` array constructor, one for all arrays or one for each key.
+            Keys should be 'm/z array', 'intensity array', 'charge array'.
+
+        encoding : str, optional
+            File encoding.
+
+        Returns
+        -------
+
+        out : MS1
+            The reader object.
+        """
         super(MS1, self).__init__(source, use_header=use_header, convert_arrays=convert_arrays, dtype=dtype, encoding=encoding,
             mode='r', parser_func=self._read, pass_file=False, args=(), kwargs={})
-        # aux.FileReader.__init__(self, source, 'r', self._read, False, (), {}, encoding)
-        # MS1Base.__init__(self, source, use_header, convert_arrays, dtype)
-        # self.encoding = encoding
 
     @aux._keepstate_method
     def _read_header(self):
         return self._read_header_lines(self._source)
 
-    def _read_spectrum(self, firstline):
-        return self._read_spectrum_lines(self._source, firstline)
-
     def _read(self):
-        reading_spectrum = False
-        params = {}
-        masses = []
-        intensities = []
-        if self._use_header:
-            params.update(self.header)
+        def get_next_spectrum():
+            return self._read_spectrum_lines(self._source)
 
-        for line in self._source:
-            sline = line.strip().split(None, 2)
-            if not sline:
-                continue
-            if not reading_spectrum:
-                if sline[0] == 'S':
-                    reading_spectrum = True
-                    self._handle_S(line, sline, params)
-                # otherwise we are not interested; do nothing, just move along
-            else:
-                if not sline:
-                    pass
-                elif sline[0] == 'S':
-                    yield self._make_scan(params, masses, intensities)
-                    params = dict(self.header) if self._use_header else {}
-                    masses = []
-                    intensities = []
-                    self._handle_S(line, sline, params)
-                else:
-                    if sline[0] == 'I':  # spectrum-specific parameters!
-                        self._handle_I(line, sline, params)
-                    elif sline[0] == 'Z':  # MS2-specific charge state guess
-                        self._handle_Z(line, sline, params)
-                    elif sline[0] == 'D':  # MS2-specific analyzer annotation
-                        self._handle_D(line, sline, params)
-                    else:  # this must be a peak list
-                        try:
-                            masses.append(float(sline[0]))            # this may cause
-                            intensities.append(float(sline[1]))       # exceptions...
-                        except ValueError:
-                            raise aux.PyteomicsError(
-                                'Error when parsing %s. Line: %s' % (self._source_name, line))
-                        except IndexError:
-                            pass
-
-        yield self._make_scan(params, masses, intensities)
+        for spectrum in iter(get_next_spectrum, None):
+            yield spectrum
 
 
 class IndexedMS1(MS1Base, aux.TaskMappingMixin, aux.TimeOrderedIndexedReaderMixin, aux.IndexedTextReader):
@@ -248,10 +301,8 @@ class IndexedMS1(MS1Base, aux.TaskMappingMixin, aux.TimeOrderedIndexedReaderMixi
     If created using a file object, it needs to be opened in binary mode.
 
     When iterated, :py:class:`IndexedMS1` object yields spectra one by one.
-    Each 'spectrum' is a :py:class:`dict` with four keys: 'm/z array',
-    'intensity array', 'charge array' and 'params'. 'm/z array' and
-    'intensity array' store :py:class:`numpy.ndarray`'s of floats,
-    'charge array' is a masked array (:py:class:`numpy.ma.MaskedArray`) of ints,
+    Each 'spectrum' is a :py:class:`dict` with three keys: 'm/z array', 'intensity array' and 'params'.
+    'm/z array' and 'intensity array' store :py:class:`numpy.ndarray`'s of floats,
     and 'params' stores a :py:class:`dict` of parameters (keys and values are
     :py:class:`str`, keys corresponding to MS1).
 
@@ -274,29 +325,53 @@ class IndexedMS1(MS1Base, aux.TaskMappingMixin, aux.TimeOrderedIndexedReaderMixi
     label = r'^[\n]?S\s+(\S+)'
 
     def __init__(self, source=None, use_header=False, convert_arrays=True, dtype=None, encoding='utf-8', _skip_index=False, **kwargs):
+        """
+        Create an :py:class:`IndexedMS1` (binary-mode) reader for a given MS1 file.
+
+        Parameters
+        ----------
+
+        source : str or file or None, optional
+            A file object (or file name) with data in MS1 format. Default is
+            :py:const:`None`, which means read standard input.
+
+            .. note :: If a file object is given, it must be opened in binary mode.
+
+        use_header : bool, optional
+            Add the info from file header to each dict. Spectrum-specific parameters
+            override those from the header in case of conflict.
+            Default is :py:const:`True`.
+
+        convert_arrays : one of {0, 1, 2}, optional
+            If `0`, m/z, intensities and (possibly) charges will be returned as regular lists.
+            If `1`, they will be converted to regular :py:class:`numpy.ndarray`'s.
+            If `2`, charges will be reported as a masked array (default).
+            The default option is the slowest. `1` and `2` require :py:mod:`numpy`.
+
+        dtype : type or str or dict, optional
+            dtype argument to :py:mod:`numpy` array constructor, one for all arrays or one for each key.
+            Keys should be 'm/z array', 'intensity array', 'charge array'.
+
+        encoding : str, optional
+            File encoding.
+
+        block_size : int, optinal
+            Size of the chunk (in bytes) used to parse the file when creating the byte offset index.
+
+        Returns
+        -------
+
+        out : IndexedMS1
+            The reader object.
+        """
         super(IndexedMS1, self).__init__(source, use_header=use_header, convert_arrays=convert_arrays, dtype=dtype, encoding=encoding,
-            parser_func=self._read, pass_file=False, args=(), kwargs={}, _skip_index=_skip_index)
-        # aux.TimeOrderedIndexedReaderMixin.__init__(self, source, self._read, False, (), {}, encoding,
-        #     block_size, _skip_index=_skip_index)
-        # MS1Base.__init__(self, source, use_header, convert_arrays, dtype)
+            parser_func=self._read, pass_file=False, args=(), kwargs={}, _skip_index=_skip_index, **kwargs)
 
     def __reduce_ex__(self, protocol):
         return (self.__class__,
-            (self._source_init, False, self._convert_arrays,
-                self._read_charges, self._dtype_dict, self.encoding, self.block_size, True),
+            (self._source_init, False, self._convert_arrays, None, self.encoding, True),
             self.__getstate__())
 
-    def __getstate__(self):
-        state = super(IndexedMS1, self).__getstate__()
-        state['use_header'] = self._use_header
-        state['header'] = self._header
-        return state
-
-    def __setstate__(self, state):
-        super(IndexedMS1, self).__setstate__(state)
-        self._use_header = state['use_header']
-        self._header = state['header']
-
     @aux._keepstate_method
     def _read_header(self):
         try:
@@ -367,10 +442,11 @@ def read(*args, **kwargs):
         override those from the header in case of conflict.
         Default is :py:const:`False`.
 
-    convert_arrays : bool, optional
-        If :py:const:`False`, m/z and intensities will be returned as regular lists.
-        If :py:const:`True` (default), they will be converted to regular :py:class:`numpy.ndarray`'s.
-        Conversion requires :py:mod:`numpy`.
+    convert_arrays : one of {0, 1, 2}, optional
+        If `0`, m/z, intensities and (possibly) charges will be returned as regular lists.
+        If `1`, they will be converted to regular :py:class:`numpy.ndarray`'s.
+        If `2`, charges will be reported as a masked array (default).
+        The default option is the slowest. `1` and `2` require :py:mod:`numpy`.
 
     dtype : type or str or dict, optional
         dtype argument to :py:mod:`numpy` array constructor, one for all arrays or one for each key.
