@@ -117,7 +117,7 @@ STANDARD_ARRAYS = set([
 ])
 
 
-class MzML(xml.ArrayConversionMixin, aux.TimeOrderedIndexedReaderMixin, xml.MultiProcessingXML, xml.IndexSavingXML):
+class MzML(aux.BinaryArrayConversionMixin, aux.TimeOrderedIndexedReaderMixin, xml.MultiProcessingXML, xml.IndexSavingXML):
     """Parser class for mzML files."""
     file_format = 'mzML'
     _root_element = 'mzML'
@@ -129,6 +129,7 @@ class MzML(xml.ArrayConversionMixin, aux.TimeOrderedIndexedReaderMixin, xml.Mult
 
     def __init__(self, *args, **kwargs):
         self.decode_binary = kwargs.pop('decode_binary', True)
+        self._referenceable_param_groups = {}
         super(MzML, self).__init__(*args, **kwargs)
 
     def __getstate__(self):
@@ -140,6 +141,19 @@ class MzML(xml.ArrayConversionMixin, aux.TimeOrderedIndexedReaderMixin, xml.Mult
         super(MzML, self).__setstate__(state)
         self.decode_binary = state['decode_binary']
 
+    def _handle_referenceable_param_group(self, param_group_ref, **kwargs):
+        ref_name = param_group_ref.attrib['ref']
+        if ref_name not in self._referenceable_param_groups:
+            params = self._referenceable_param_groups[ref_name] = self._retrieve_param_group(ref_name)
+            return params
+        return self._referenceable_param_groups[ref_name]
+
+    @xml._keepstate
+    def _retrieve_param_group(self, ref_name):
+        group = self.get_by_id(ref_name)
+        group.pop("id", None)
+        return [xml._XMLParam(k, v, None) for k, v in group.items()]
+
     def _detect_array_name(self, info):
         """Determine what the appropriate name for this
         array is by inspecting the available param-based
@@ -322,7 +336,10 @@ class MzML(xml.ArrayConversionMixin, aux.TimeOrderedIndexedReaderMixin, xml.Mult
         intkeys = {'ms level'}
         for k in intkeys:
             if k in info:
-                info[k] = int(info[k])
+                try:
+                    info[k] = int(info[k])
+                except (ValueError, TypeError):
+                    pass
         return info
 
     def _retrieve_refs(self, info, **kwargs):
@@ -343,7 +360,7 @@ class MzML(xml.ArrayConversionMixin, aux.TimeOrderedIndexedReaderMixin, xml.Mult
         return scan['scanList']['scan'][0]['scan start time']
 
 
-def read(source, read_schema=False, iterative=True, use_index=False, dtype=None, huge_tree=False):
+def read(source, read_schema=False, iterative=True, use_index=False, dtype=None, huge_tree=False, decode_binary=True):
     """Parse `source` and iterate through spectra.
 
     Parameters
@@ -389,7 +406,8 @@ def read(source, read_schema=False, iterative=True, use_index=False, dtype=None,
     """
 
     return MzML(source, read_schema=read_schema, iterative=iterative,
-        use_index=use_index, dtype=dtype, huge_tree=huge_tree)
+                use_index=use_index, dtype=dtype, huge_tree=huge_tree,
+                decode_binary=decode_binary)
 
 def iterfind(source, path, **kwargs):
     """Parse `source` and yield info on elements with specified local
@@ -453,17 +471,22 @@ class PreIndexedMzML(MzML):
     """Parser class for mzML files, subclass of :py:class:`MzML`.
     Uses byte offsets listed at the end of the file for quick access to spectrum elements.
     """
-    def _build_index(self):
+    def build_byte_index(self):
         """
-        Build up a `dict` of `dict` of offsets for elements. Calls :meth:`_find_index_list`
-        and assigns the return value to :attr:`_offset_index`
+        Build up a :class:`HierarchicalOffsetIndex` of offsets for elements. Calls :meth:`_find_index_list` or
+        falls back on regular :class:`MzML` indexing.
+
+        Returns
+        -------
+
+        out : HierarchicalOffsetIndex
         """
         index = self._find_index_list()
         if index:
-            self._offset_index = index
+            return index
         else:
             warnings.warn('Could not extract the embedded offset index. Falling back to default indexing procedure.')
-            super(PreIndexedMzML, self)._build_index()
+            return super(PreIndexedMzML, self).build_byte_index()
 
     @xml._keepstate
     def _iterparse_index_list(self, offset):
