@@ -32,7 +32,8 @@ except ImportError:
     np = None
 
 try:
-    from psims.controlled_vocabulary.controlled_vocabulary import (load_psimod, load_xlmod, load_gno, obo_cache)
+    from psims.controlled_vocabulary.controlled_vocabulary import (load_psimod, load_xlmod, load_gno, obo_cache, load_unimod)
+    _has_psims = True
 except ImportError:
     def _needs_psims(name):
         raise ImportError("Loading %s requires the `psims` library. To access it, please install `psims`" % name)
@@ -40,9 +41,11 @@ except ImportError:
     load_psimod = partial(_needs_psims, 'PSIMOD')
     load_xlmod = partial(_needs_psims, 'XLMOD')
     load_gno = partial(_needs_psims, 'GNO')
+    load_unimod = partial(_needs_psims, 'UNIMOD')
     obo_cache = None
+    _has_psims = False
 
-_water_mass = calculate_mass("H2O")
+_WATER_MASS = calculate_mass(formula="H2O")
 
 std_aa_mass = std_aa_mass.copy()
 std_aa_mass['X'] = 0
@@ -99,6 +102,14 @@ class TagTypeEnum(Enum):
     group_placeholder = 999
 
 
+class ModificationTagStyle(Enum):
+    Unset = 0
+    ShortId = 1
+    LongId = 2
+    ShortName = 3
+    LongName = 4
+
+
 _sentinel = object()
 
 
@@ -255,39 +266,34 @@ class InformationTag(TagBase):
         return str(self.value)
 
 
-class MassModification(TagBase):
-    '''A modification defined purely by a signed mass shift in Daltons.
-
-    The value of a :class:`MassModification` is always a :class:`float`
-    '''
-    __slots__ = ('_significant_figures', )
-
-    prefix_name = "Obs"
-
-    def __init__(self, value, extra=None, group_id=None):
-        if isinstance(value, str):
-            sigfigs = len(value.split('.')[-1].rstrip('0'))
-        else:
-            sigfigs = 4
-        self._significant_figures = sigfigs
-        super(MassModification, self).__init__(
-            TagTypeEnum.massmod, float(value), extra, group_id)
-
-    def _format_main(self):
-        if self.value >= 0:
-            return ('+{0:0.{1}f}'.format(self.value, self._significant_figures)).rstrip('0').rstrip('.')
-        else:
-            return ('{0:0.{1}f}'.format(self.value, self._significant_figures)).rstrip('0').rstrip('.')
-
-    @property
-    def mass(self):
-        return self.value
-
-
 class ModificationResolver(object):
     def __init__(self, name, **kwargs):
-        self.name = name
+        self.name = name.lower()
+        self.symbol = self.name[0]
         self._database = None
+        self._cache = {}
+
+    def clear_cache(self):
+        """Clear the modification definition cache"""
+        self._cache.clear()
+
+    def enable_caching(self, flag=True):
+        """
+        Enable or disable caching of modification definitions.
+
+        If `flag` is :const:`False`, this will also dispose of any
+        existing cached values.
+
+        Parameters
+        ----------
+        flag : :class:`bool`
+            Whether or not to disable the cache
+        """
+        if flag:
+            if not self._cache:
+                self._cache = {}
+        else:
+            self._cache = None
 
     def load_database(self):
         raise NotImplementedError()
@@ -302,9 +308,50 @@ class ModificationResolver(object):
     def database(self, database):
         self._database = database
 
-    def resolve(self, name=None, id=None, **kwargs):
+    def parse_identifier(self, identifier):
+        """Parse a string that is either a CV prefixed identifier or name.
+
+        Parameters
+        ----------
+        identifier : str
+            The identifier string to parse, removing CV prefix as needed.
+
+        Returns
+        -------
+        name : str, optional
+            A textual identifier embedded in the qualified identifier, if any, otherwise
+            :const:`None`.
+        id : int, optional
+            An integer ID embedded in the qualified identifier, if any, otherwise
+            :const:`None`.
+        """
+        tokens = identifier.split(":", 1)
+        if len(tokens) > 1:
+            prefix = tokens[0].lower()
+            if prefix == self.name or prefix == self.symbol:
+                identifier = tokens[1]
+
+        if identifier.isdigit():
+            id = int(identifier)
+            name = None
+        else:
+            name = identifier
+            id = None
+        return name, id
+
+    def _resolve_impl(self, name=None, id=None, **kwargs):
         raise NotImplementedError()
 
+    def resolve(self, name=None, id=None, **kwargs):
+        if self._cache is None:
+            return self._resolve_impl(name, id, **kwargs)
+        cache_key = (name, id, frozenset(kwargs.items()))
+        if cache_key in self._cache:
+            return self._cache[cache_key].copy()
+        value = self._resolve_impl(name, id, **kwargs)
+        self._cache[cache_key] = value
+        return  value.copy()
+
     def __call__(self, name=None, id=None, **kwargs):
         return self.resolve(name, id, **kwargs)
 
@@ -325,9 +372,11 @@ class UnimodResolver(ModificationResolver):
         self.strict = kwargs.get("strict", True)
 
     def load_database(self):
+        if _has_psims:
+            return obo_cache.resolve("http://www.unimod.org/obo/unimod.obo")
         return Unimod()
 
-    def resolve(self, name=None, id=None, **kwargs):
+    def _resolve_impl(self, name=None, id=None, **kwargs):
         strict = kwargs.get("strict", self.strict)
         exhaustive = kwargs.get("exhaustive", True)
         if name is not None:
@@ -346,7 +395,7 @@ class UnimodResolver(ModificationResolver):
             if not defn:
                 raise KeyError(name)
         elif id is not None:
-            defn = self.database.by_id(id)
+            defn = self.database[id]
             if not defn:
                 raise KeyError(id)
         else:
@@ -357,15 +406,20 @@ class UnimodResolver(ModificationResolver):
                 'name': defn['title'],
                 'id': defn['record_id'],
                 'mass': defn['mono_mass'],
-                'provider': self.name
+                'provider': self.name,
+                "source": self
             }
         else:
+            name = defn.ex_code_name
+            if not name:
+                name = defn.code_name
             return {
                 "composition": defn.composition,
-                "name": defn.ex_code_name,
+                "name": name,
                 "id": defn.id,
                 "mass": defn.monoisotopic_mass,
-                "provider": self.name
+                "provider": self.name,
+                "source": self
             }
 
 
@@ -377,7 +431,7 @@ class PSIModResolver(ModificationResolver):
     def load_database(self):
         return load_psimod()
 
-    def resolve(self, name=None, id=None, **kwargs):
+    def _resolve_impl(self, name=None, id=None, **kwargs):
         if name is not None:
             defn = self.database[name]
         elif id is not None:
@@ -409,7 +463,8 @@ class PSIModResolver(ModificationResolver):
             'composition': composition,
             'name': defn.name,
             'id': defn.id,
-            'provider': self.name
+            'provider': self.name,
+            "source": self
         }
 
 
@@ -421,7 +476,7 @@ class XLMODResolver(ModificationResolver):
     def load_database(self):
         return load_xlmod()
 
-    def resolve(self, name=None, id=None, **kwargs):
+    def _resolve_impl(self, name=None, id=None, **kwargs):
         if name is not None:
             defn = self.database[name]
         elif id is not None:
@@ -442,7 +497,8 @@ class XLMODResolver(ModificationResolver):
             'composition': composition,
             'name': defn.name,
             'id': defn.id,
-            'provider': self.name
+            'provider': self.name,
+            "source": self
         }
 
 # TODO: Implement resolve walking up the graph to get the mass. Can't really
@@ -531,7 +587,7 @@ class GNOResolver(ModificationResolver):
         if not match:
             return None
         # This will have a small mass error.
-        rough_mass = float(match.group(1)) - _water_mass
+        rough_mass = float(match.group(1)) - _WATER_MASS
         if raw_mass is not None and abs(rough_mass - raw_mass) < 1:
             return raw_mass
         warnings.warn(
@@ -539,7 +595,7 @@ class GNOResolver(ModificationResolver):
              "Only a rough approximation is available.") % (term, ))
         return rough_mass
 
-    def resolve(self, name=None, id=None, **kwargs):
+    def _resolve_impl(self, name=None, id=None, **kwargs):
         if name is not None:
             term = self.database[name]
         elif id is not None:
@@ -554,7 +610,8 @@ class GNOResolver(ModificationResolver):
             "provider": self.name,
             "composition": composition,
             "monosaccharides": monosaccharides,
-            "mass": self.get_mass_from_term(term, raw_mass)
+            "mass": self.get_mass_from_term(term, raw_mass),
+            "source": self
         }
         return rec
 
@@ -568,7 +625,28 @@ class GenericResolver(ModificationResolver):
     def load_database(self):
         return None
 
-    def resolve(self, name=None, id=None, **kwargs):
+    def parse_identifier(self, identifier):
+        """Parse a string that is either a CV prefixed identifier or name.
+
+        Does no parsing as a :class:`GenericModification` is never qualified.
+
+        Parameters
+        ----------
+        identifier : str
+            The identifier string to parse, removing CV prefix as needed.
+
+        Returns
+        -------
+        name : str, optional
+            A textual identifier embedded in the qualified identifier, if any, otherwise
+            :const:`None`.
+        id : int, optional
+            An integer ID embedded in the qualified identifier, if any, otherwise
+            :const:`None`.
+        """
+        return identifier, None
+
+    def _resolve_impl(self, name=None, id=None, **kwargs):
         defn = None
         for resolver in self.resolvers:
             try:
@@ -599,12 +677,28 @@ class ModificationBase(TagBase):
     '''
 
     _tag_type = None
-    __slots__ = ('_definition', )
+    __slots__ = ('_definition', 'style')
 
-    def __init__(self, value, extra=None, group_id=None):
+    def __init__(self, value, extra=None, group_id=None, style=None):
+        if style is None:
+            style = ModificationTagStyle.Unset
         super(ModificationBase, self).__init__(
             self._tag_type, value, extra, group_id)
         self._definition = None
+        self.style = style
+
+    def __reduce__(self):
+        return self.__class__, (self.value, self.extra, self.group_id, self.style), self.__getstate__()
+
+    def __getstate__(self):
+        if self._definition is None:
+            return None
+        state = self._definition.copy()
+        state['source'] = None
+        return state
+
+    def __setstate__(self, state):
+        self._definition = state
 
     def __eq__(self, other):
         if isinstance(other, ModificationToken):
@@ -688,29 +782,83 @@ class ModificationBase(TagBase):
         self._definition = definition
 
     def _format_main(self):
-        return "{self.prefix_name}:{self.value}".format(self=self)
-
-    def _parse_identifier(self):
-        tokens = self.value.split(":", 1)
-        if len(tokens) > 1:
-            value = tokens[1]
+        if self.style == ModificationTagStyle.Unset or self.style is None:
+            return "{self.prefix_name}:{self.value}".format(self=self)
+        elif self.style == ModificationTagStyle.LongId:
+            return "{self.prefix_name}:{self.id}".format(self=self)
+        elif self.style == ModificationTagStyle.ShortId:
+            return "{self.short_prefix}:{self.id}".format(self=self)
+        elif self.style == ModificationTagStyle.LongName:
+            return "{self.prefix_name}:{self.name}".format(self=self)
+        elif self.style == ModificationTagStyle.ShortName:
+            return "{self.short_prefix}:{self.name}".format(self=self)
         else:
-            value = self.value
-        if value.isdigit():
-            id = int(value)
-            name = None
-        else:
-            name = value
-            id = None
-        return name, id
+            warnings.warn("Unknown formatting style {!r}".format(self.style))
+            return "{self.prefix_name}:{self.value}".format(self=self)
 
     def resolve(self):
         '''Find the term and return it's properties
         '''
-        keys = self._parse_identifier()
+        keys = self.resolver.parse_identifier(self.value)
         return self.resolver(*keys)
 
 
+class MassModification(TagBase):
+    '''A modification defined purely by a signed mass shift in Daltons.
+
+    The value of a :class:`MassModification` is always a :class:`float`
+    '''
+    __slots__ = ('_significant_figures', )
+
+    prefix_name = "Obs"
+
+    def __init__(self, value, extra=None, group_id=None):
+        if isinstance(value, str):
+            sigfigs = len(value.split('.')[-1].rstrip('0'))
+        else:
+            sigfigs = 4
+        self._significant_figures = sigfigs
+        super(MassModification, self).__init__(
+            TagTypeEnum.massmod, float(value), extra, group_id)
+
+    def _format_main(self):
+        if self.value >= 0:
+            return ('+{0:0.{1}f}'.format(self.value, self._significant_figures)).rstrip('0').rstrip('.')
+        else:
+            return ('{0:0.{1}f}'.format(self.value, self._significant_figures)).rstrip('0').rstrip('.')
+
+    @property
+    def provider(self):
+        return None
+
+    @property
+    def id(self):
+        return self._format_main()
+
+    @property
+    def key(self):
+        '''Get a safe-to-hash-and-compare :class:`ModificationToken`
+        representing this modification without tag-like properties.
+
+        Returns
+        --------
+        ModificationToken
+        '''
+        return ModificationToken(self.value, self.id, self.provider, self.__class__)
+
+    @property
+    def mass(self):
+        return self.value
+
+    def __eq__(self, other):
+        if isinstance(other, ModificationToken):
+            return other == self
+        return super(MassModification, self).__eq__(other)
+
+    def __hash__(self):
+        return hash((self.id, self.provider))
+
+
 class FormulaModification(ModificationBase):
     prefix_name = "Formula"
 
@@ -882,9 +1030,9 @@ class GenericModification(ModificationBase):
         partial(UnimodModification.resolver, strict=False)
     ])
 
-    def __init__(self, value, extra=None, group_id=None):
+    def __init__(self, value, extra=None, group_id=None, style=None):
         super(GenericModification, self).__init__(
-            value, extra, group_id)
+            value, extra, group_id, style)
 
     def _format_main(self):
         return self.value
@@ -893,7 +1041,7 @@ class GenericModification(ModificationBase):
         '''Find the term, searching through all available vocabularies and
         return the first match's properties
         '''
-        keys = self._parse_identifier()
+        keys = self.resolver.parse_identifier(self.value)
         defn = self.resolver(*keys)
         if defn is not None:
             return defn
@@ -960,7 +1108,7 @@ class ModificationToken(object):
     def __eq__(self, other):
         if other is None:
             return False
-        if isinstance(other, (ModificationToken, ModificationBase)):
+        if isinstance(other, (ModificationToken, ModificationBase, MassModification)):
             return self.id == other.id and self.provider == other.provider
         return False
 
@@ -1103,8 +1251,13 @@ def process_tag_tokens(tokens):
         if prefix is None:
             main_tag = GenericModification(''.join(value))
         else:
-            tag_type = TagBase.find_by_tag(prefix)
-            main_tag = tag_type(value)
+            try:
+                tag_type = TagBase.find_by_tag(prefix)
+                main_tag = tag_type(value)
+            except KeyError:
+                main_tag_str = ''.join(main_tag)
+                main_tag = GenericModification(main_tag_str)
+
     if len(parts) > 1:
         extras = []
         for part in parts[1:]:
@@ -1120,12 +1273,67 @@ def process_tag_tokens(tokens):
                 else:
                     extras.append(GenericModification(''.join(value)))
             else:
-                tag_type = TagBase.find_by_tag(prefix)
-                extras.append(tag_type(value))
+                try:
+                    tag_type = TagBase.find_by_tag(prefix)
+                    extra_tag = tag_type(value)
+                except KeyError:
+                    part_str = ''.join(part)
+                    extra_tag = GenericModification(part_str)
+                extras.append(extra_tag)
         main_tag.extra = extras
     return main_tag
 
 
+class ModificationTarget(object):
+    def __init__(self, aa, n_term=False, c_term=False):
+        self.aa = aa
+        self.n_term = n_term
+        self.c_term = c_term
+
+    def __eq__(self, other):
+        if isinstance(other, str):
+            return str(self) == other
+        else:
+            return (
+                self.aa == other.aa
+                and self.n_term == other.n_term
+                and self.c_term == other.c_term
+            )
+
+    def __ne__(self, other):
+        if isinstance(other, str):
+            return str(self) != other
+        else:
+            return (
+                self.aa != other.aa
+                or self.n_term != other.n_term
+                or self.c_term != other.c_term
+            )
+
+    def __hash__(self):
+        return hash(str(self))
+
+    def __str__(self):
+        buffer = []
+        if self.n_term:
+            buffer.append('N-term')
+        if self.c_term:
+            buffer.append('C-term')
+        if self.aa:
+            buffer.append(self.aa)
+        return ':'.join(buffer)
+
+    def __repr__(self):
+        return str(self)
+
+    def is_valid(self, aa, n_term, c_term):
+        if (n_term and self.n_term) or (c_term and self.c_term):
+            if (self.aa and aa == self.aa) or self.aa is None:
+                return True
+            return False
+        return self.aa == aa or self.aa is None
+
+
 class ModificationRule(object):
     '''Define a fixed modification rule which dictates a modification tag is
     always applied at one or more amino acid residues.
@@ -1142,6 +1350,53 @@ class ModificationRule(object):
     def __init__(self, modification_tag, targets=None):
         self.modification_tag = modification_tag
         self.targets = targets
+        self._validate_targets()
+
+    def is_valid(self, aa, n_term, c_term):
+        return any(target.is_valid(aa, n_term, c_term) for target in self.targets)
+
+    def _validate_targets(self):
+        validated_targets = []
+        if self.targets is None:
+            self.targets = []
+        elif not isinstance(self.targets, list):
+            self.targets = [self.targets]
+        for target in self.targets:
+            if target in VALID_AA:
+                validated_targets.append(ModificationTarget(target, False, False))
+            elif target in ("N-term", "C-term"):
+                n_term = target == "N-term"
+                c_term = target == "C-term"
+                validated_targets.append(ModificationTarget(None, n_term, c_term))
+            elif target.startswith(("N-term:", "C-term:")):
+                tokens = target.split(":")
+                if len(tokens) == 2:
+                    if tokens[1] in VALID_AA:
+                        n_term = tokens[0] == "N-term"
+                        c_term = tokens[0] == "C-term"
+                        validated_targets.append(ModificationTarget(tokens[1], n_term, c_term))
+                    else:
+                        raise PyteomicsError(
+                            "Modification rule {0} has an invalid amino acid specific terminal target {2} in {1}".format(
+                                self,
+                                target,
+                                tokens[1]
+                            )
+                        )
+                else:
+                    raise PyteomicsError(
+                        "Modification rule {0} has an empty amino acid specific terminal target {1}".format(
+                            self, target
+                        )
+                    )
+            else:
+                raise PyteomicsError(
+                    "Modification rule {0} has an invalid target {1}".format(
+                        self, target
+                    )
+                )
+
+        self.targets = validated_targets
 
     def __eq__(self, other):
         if other is None:
@@ -1152,7 +1407,7 @@ class ModificationRule(object):
         return not self == other
 
     def __str__(self):
-        targets = ','.join(self.targets)
+        targets = ','.join(map(str, self.targets))
         return "<[{self.modification_tag}]@{targets}>".format(self=self, targets=targets)
 
     def __repr__(self):
@@ -1484,6 +1739,7 @@ class ParserStateEnum(Enum):
     inter_chain_cross_link_start = 20
     chimeric_start = 21
     interval_initial = 22
+    post_global_terminal = 23
     done = 999
 
 
@@ -1503,6 +1759,7 @@ POST_TAG_AFTER = ParserStateEnum.post_tag_after
 UNLOCALIZED_COUNT = ParserStateEnum.unlocalized_count
 POST_GLOBAL = ParserStateEnum.post_global
 POST_GLOBAL_AA = ParserStateEnum.post_global_aa
+POST_GLOBAL_TERM = ParserStateEnum.post_global_terminal
 POST_INTERVAL_TAG = ParserStateEnum.post_interval_tag
 CHARGE_START = ParserStateEnum.charge_state_start
 CHARGE_NUMBER = ParserStateEnum.charge_state_number
@@ -1511,6 +1768,7 @@ ADDUCT_END = ParserStateEnum.charge_state_adduct_end
 DONE = ParserStateEnum.done
 
 VALID_AA = set("QWERTYIPASDFGHKLCVNMXUOJZB")
+TERMINAL_SPEC_CHARS = set('N-term') | set('C-term') | set("ncT: ")
 
 def parse(sequence):
     '''Tokenize a ProForma sequence into a sequence of amino acid+tag positions, and a
@@ -1553,7 +1811,7 @@ def parse(sequence):
     current_tag = TagParser()
     current_interval = None
     current_unlocalized_count = NumberParser()
-    current_aa_targets = TokenBuffer()
+    current_aa_targets = StringParser()
 
     charge_buffer = None
     adduct_buffer = None
@@ -1743,13 +2001,13 @@ def parse(sequence):
                 depth = 1
                 tag = current_tag()[0]
                 multiplicity = current_unlocalized_count()
-                for i in range(multiplicity):
+                for _ in range(multiplicity):
                     unlocalized_modifications.append(tag)
             elif c == '?':
                 state = BEFORE
                 tag = current_tag()[0]
                 multiplicity = current_unlocalized_count()
-                for i in range(multiplicity):
+                for _ in range(multiplicity):
                     unlocalized_modifications.append(tag)
             else:
                 raise ProFormaError(
@@ -1762,14 +2020,25 @@ def parse(sequence):
                     ("Error In State {state}, fixed modification detected without "
                      "target amino acids found at index {i}").format(**locals()), i, state)
         elif state == POST_GLOBAL_AA:
-            if c in VALID_AA:
+            if c in VALID_AA or c in TERMINAL_SPEC_CHARS:
                 current_aa_targets.append(c)
             elif c == ',':
                 # the next character should be another amino acid
-                pass
+                current_aa_targets.bound()
             elif c == '>':
-                fixed_modifications.append(
-                    ModificationRule(current_tag()[0], current_aa_targets()))
+                try:
+                    v = current_aa_targets()
+                    fixed_modifications.append(
+                        ModificationRule(current_tag()[0], v))
+                except PyteomicsError as err:
+                    raise ProFormaError(
+                        (
+                            "Error In State {state}, fixed modification detected invalid "
+                            "target found at index {i}: {err}"
+                        ).format(state=state, i=i, err=err),
+                        i,
+                        state,
+                    )
                 state = BEFORE
             else:
                 raise ProFormaError(
@@ -1925,8 +2194,6 @@ class _ProFormaProperty(object):
         return template.format(self=self)
 
 
-_WATER_MASS = calculate_mass(formula="H2O")
-
 class ProForma(object):
     '''Represent a parsed ProForma sequence.
 
@@ -1985,6 +2252,9 @@ class ProForma(object):
     def __repr__(self):
         return "{self.__class__.__name__}({self.sequence}, {self.properties})".format(self=self)
 
+    def __len__(self):
+        return len(self.sequence)
+
     def __getitem__(self, i):
         if isinstance(i, slice):
             props = self.properties.copy()
@@ -1996,6 +2266,13 @@ class ProForma(object):
                     continue
                 ivs.append(iv)
             props['intervals'] = ivs
+
+            if not (i.start is None or i.start == 0):
+                props['n_term'] = None
+            n = len(self)
+            if not (i.stop is None or i.stop >= n):
+                props['c_term'] = None
+
             return self.__class__(self.sequence[i], props)
         else:
             return self.sequence[i]
@@ -2031,19 +2308,20 @@ class ProForma(object):
         mass = 0.0
 
         fixed_modifications = self.properties['fixed_modifications']
-        fixed_rules = {}
-        for rule in fixed_modifications:
-            for aa in rule.targets:
-                fixed_rules[aa] = rule.modification_tag.mass
 
-        for position in self.sequence:
+        n_term_v = 0
+        c_term_v = len(self) - 1
+        for i, position in enumerate(self.sequence):
             aa = position[0]
             try:
                 mass += std_aa_mass[aa]
             except KeyError:
                 warnings.warn("%r does not have an exact mass" % (aa, ))
-            if aa in fixed_rules:
-                mass += fixed_rules[aa]
+            n_term = i == n_term_v
+            c_term = i == c_term_v
+            for rule in fixed_modifications:
+                if rule.is_valid(aa, n_term, c_term):
+                    mass += rule.modification_tag.mass
             tags = position[1]
             if tags:
                 for tag in tags:
@@ -2128,10 +2406,6 @@ class ProForma(object):
         mass += ion_shift
 
         fixed_modifications = self.properties['fixed_modifications']
-        fixed_rules = {}
-        for rule in fixed_modifications:
-            for aa in rule.targets:
-                fixed_rules[aa] = rule.modification_tag.mass
 
         intervals = self.intervals
         if intervals:
@@ -2165,8 +2439,12 @@ class ProForma(object):
 
         if not reverse:
             iterator = (iter(range(0, n - 1)))
+            n_term_v = 0
+            c_term_v = n - 1
         else:
             iterator = (reversed(range(1, n)))
+            n_term_v = n - 1
+            c_term_v = 0
 
         for i in iterator:
             position = self.sequence[i]
@@ -2177,8 +2455,11 @@ class ProForma(object):
             except KeyError:
                 warnings.warn("%r does not have an exact mass" % (aa, ))
 
-            if aa in fixed_rules:
-                mass += fixed_rules[aa]
+            n_term = i == n_term_v
+            c_term = i == c_term_v
+            for rule in fixed_modifications:
+                if rule.is_valid(aa, n_term, c_term):
+                    mass += rule.modification_tag.mass
 
             tags = position[1]
             if tags:
