@@ -39,7 +39,6 @@ import numpy as np
 from .auxiliary import FileReader, PyteomicsError, basestring, _file_obj, HierarchicalOffsetIndex
 from .auxiliary import unitint, unitfloat, unitstr, cvstr
 from .auxiliary import _keepstate_method as _keepstate
-from .auxiliary import BinaryDataArrayTransformer
 from .auxiliary import TaskMappingMixin, IndexedReaderMixin, IndexSavingMixin
 
 try:  # Python 2.7
@@ -372,9 +371,13 @@ class XML(FileReader):
         # return {cvstr(attribs['name'], accession, unit_accesssion): value}
         return _XMLParam(cvstr(attribs['name'], accession, unit_accesssion), value, _local_name(element))
 
+    def _handle_referenceable_param_group(self, param_group_ref, **kwargs):
+        raise NotImplementedError()
+        return []
+
     def _find_immediate_params(self, element, **kwargs):
         return element.xpath(
-            './*[local-name()="cvParam" or local-name()="userParam" or local-name()="UserParam"]')
+            './*[local-name()="cvParam" or local-name()="userParam" or local-name()="UserParam" or local-name()="referenceableParamGroupRef"]')
 
     def _insert_param(self, info_dict, param):
         key = param.name
@@ -410,6 +413,8 @@ class XML(FileReader):
         schema_info = self.schema_info
         if name in {'cvParam', 'userParam', 'UserParam'}:
             return self._handle_param(element, **kwargs)
+        elif name == "referenceableParamGroupRef":
+            return self._handle_referenceable_param_group(element, **kwargs)
 
         info = dict(element.attrib)
         # process subelements
@@ -420,6 +425,8 @@ class XML(FileReader):
                 if cname in {'cvParam', 'userParam', 'UserParam'}:
                     newinfo = self._handle_param(child, **kwargs)
                     params.append(newinfo)
+                elif cname == "referenceableParamGroupRef":
+                    params.extend(self._handle_referenceable_param_group(child, **kwargs))
                 else:
                     if cname not in schema_info['lists']:
                         info[cname] = self._get_info_smart(child, ename=cname, **kwargs)
@@ -430,7 +437,11 @@ class XML(FileReader):
             # handle the case where we do not want to unpack all children, but
             # *Param tags are considered part of the current entity, semantically
             for child in self._find_immediate_params(element, **kwargs):
-                params.append(self._handle_param(child, **kwargs))
+                param_or_group = self._handle_param(child, **kwargs)
+                if isinstance(param_or_group, list):
+                    params.extend(param_or_group)
+                else:
+                    params.append(param_or_group)
 
         handler = self._element_handlers.get(name)
         if handler is not None:
@@ -711,7 +722,7 @@ def xpath(tree, path, ns=None):
         if not s: return 'd:'
         return '/d:'
     new_path = re.sub(r'(\/|^)(?![\*\/])', repl, path)
-    n_s = ({'d': ns} if ns else None)
+    n_s = {'d': ns} if ns else None
     return tree.xpath(new_path, namespaces=n_s)
 
 
@@ -1062,8 +1073,7 @@ class IndexedXML(IndexedReaderMixin, XML):
                     '_default_iter_tag will be used in the index, mind the consequences.')
         super(IndexedXML, self).__init__(source, read_schema, iterative, build_id_cache, *args, **kwargs)
 
-        self._offset_index = None
-        self._build_index()
+        self._offset_index = self.build_byte_index()
 
     @property
     def default_index(self):
@@ -1090,14 +1100,18 @@ class IndexedXML(IndexedReaderMixin, XML):
         self._offset_index = state['_offset_index']
 
     @_keepstate
-    def _build_index(self):
+    def build_byte_index(self):
         """
-        Build up a `dict` of `dict` of offsets for elements. Calls :func:`find_index_list`
-        on :attr:`_source` and assigns the return value to :attr:`_offset_index`
+        Build up an index of offsets for elements.
+
+        Returns
+        -------
+
+        out : TagSpecificXMLByteIndex
         """
         if not self._indexed_tags or not self._use_index:
             return
-        self._offset_index = TagSpecificXMLByteIndex.build(
+        return TagSpecificXMLByteIndex.build(
             self._source, self._indexed_tags, self._indexed_tag_keys)
 
     @_keepstate
@@ -1166,6 +1180,10 @@ class IndexedXML(IndexedReaderMixin, XML):
             return IndexedIterfind(self, path, **kwargs)
         return Iterfind(self, path, **kwargs)
 
+    def __init_subclass__(cls, **kwargs):  # only works on Python 3.x
+        super(IndexedXML, cls).__init_subclass__(**kwargs)
+        if hasattr(cls, '_build_index'):
+            warnings.warn("The method `_build_index` has been renamed to `build_byte_index`.")
 
 class MultiProcessingXML(IndexedXML, TaskMappingMixin):
     """XML reader that feeds indexes to external processes
@@ -1197,41 +1215,7 @@ class IndexSavingXML(IndexSavingMixin, IndexedXML):
             index = self._index_class.load(f)
             if index.schema_version is None:
                 raise TypeError("Legacy Offset Index!")
-            self._offset_index = index
-
-
-class ArrayConversionMixin(BinaryDataArrayTransformer):
-    _dtype_dict = {}
-    _array_keys = ['m/z array', 'intensity array']
-
-    def __init__(self, *args, **kwargs):
-        self._dtype_dict = {None: None}
-        dtype = kwargs.pop('dtype', None)
-        if isinstance(dtype, dict):
-            self._dtype_dict.update(dtype)
-        elif dtype:
-            self._dtype_dict = {k: dtype for k in self._array_keys}
-            self._dtype_dict[None] = dtype
-        super(ArrayConversionMixin, self).__init__(*args, **kwargs)
-
-    def __getstate__(self):
-        state = super(ArrayConversionMixin, self).__getstate__()
-        state['_dtype_dict'] = self._dtype_dict
-        return state
-
-    def __setstate__(self, state):
-        super(ArrayConversionMixin, self).__setstate__(state)
-        self._dtype_dict = state['_dtype_dict']
-
-    def _convert_array(self, k, array):
-        dtype = self._dtype_dict.get(k)
-        if dtype is not None:
-            return array.astype(dtype)
-        return array
-
-    def _finalize_record_conversion(self, array, record):
-        key = record.key
-        return self._convert_array(key, array)
+            return index
 
 
 class Iterfind(object):
